#!/usr/bin/perl -w

package _NumericEntry;

BEGIN {
    use src::MymoneyUI;
};


use strict;


#use Encode;
use utf8;
#use open ':utf8', ':std';

use Gtk2;
init Gtk2;
use Gnome2;
#init Gnome;

use lib "./";
use lib::Tools;

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
    
    $self->{'DECIMALS'}=2;
    $self->{'entry'} = new Gtk2::Entry;
    $self->{'entry'}->set_max_length('0');
    $self->{'entry'}->set_visibility('1');
    $self->{'entry'}->set_editable('0');
    $self->{'entry'}->set_activates_default('0');
    #$self->{'entry'}->set_size_request(70, 24);
    $self->{'entry'}->set_has_frame('1');
    $self->{'entry'}->set_alignment(1);
    #$self->{'entry'}->SET_FLAGS('can-focus');
    $self->{'entry'}->set_events(['key_press_mask']);
    $self->{'entry'}->signal_connect("key_press_event","$class\::key_press_event",$self);
    $self->set_value(0);
    $self->{'changing_colors_by_minus'}=1;
    $self->{'color_for_minus'}="red";
    $self->{'color_for_plus'}="black";
    return $self;
};


sub get_widget {
    my $self=shift;
    return($self->{'entry'});
};

sub show {
    my $self=shift;
    $self->{'entry'}->show;
};

sub modify_bg {
    my ($self,$state,$color)=@_;
    $self->{'entry'}->modify_bg($state, Gtk2::Gdk::Color->parse($color));
};

sub changing_colors_by_minus {
    my ($self,$value)=@_;
    $self->{'changing_colors_by_minus'}=$value;
};

sub set_color_for_minus {
    my ($self,$value)=@_;
    $self->{'color_for_minus'}=$value;
};

sub set_color_for_plus {
    my ($self,$value)=@_;
    $self->{'color_for_plus'}=$value;
};

sub set_text_color {
    my ($self,$value)=@_;
    $self->{'entry'}->modify_text('normal', Gtk2::Gdk::Color->parse($value));
};

sub hide {
    my $self=shift;
    $self->{'entry'}->hide;
};

sub set_sensitive {
    my ($self,$value)=@_;
    $self->{'entry'}->set_sensitive($value);
};

sub set_value {
    my ($self,$value)=@_;
    my $DECIMALS=$self->{'DECIMALS'};
    $self->{'entry'}->set_text(Tools::HumanNums(sprintf("%01.".$DECIMALS."f",($value*1))));
    if($self->{'changing_colors_by_minus'}) {
        if($value<0) {
                $self->{'entry'}->modify_text('normal', Gtk2::Gdk::Color->parse($self->{'color_for_minus'}));
        } else {
                $self->{'entry'}->modify_text('normal', Gtk2::Gdk::Color->parse($self->{'color_for_plus'}));
        };
    };
    s/,/\./;
    s/ //g;
};

sub get_value {
    my $self=shift;
    return(Tools::FromHumanNums($self->{'entry'}->get_text));
};

sub set_size {
    my ($self,$x,$y)=@_;
    $self->{'entry'}->set_size_request($x, $y);
};

sub signal_connect {
    my $self=shift;
    my $signal=shift;
    my $class=ref($self);
    
    $self->{'entry'}->signal_connect($signal,"$class\::signal_handler",[$self,$signal,@_]);
};

sub signal_handler {
    my($entry,$data)=@_;
    my($self,$signal,$handler,$userdata)=@{$data};
    
    eval "$handler \$self,\$userdata;";
};

sub key_press_event {
    my($entry,$eventkey,$self)=@_;

    my $DECIMALS=$self->{'DECIMALS'};
    
    #print($eventkey->keyval."\n");

    # MINUS
    if(($eventkey->keyval eq '45') || ($eventkey->keyval eq '65453')) {
        $_=$self->get_value;
        if(/^[0-9]/) {
            $self->set_value("-".$_);
            $entry->set_position(1);
        } else {
            $entry->set_position(0);
        };
        return(1);
    };
    
    # PLUS
    if(($eventkey->keyval eq '43') || ($eventkey->keyval eq '65451')) {
        $_=$self->get_value;
        if(/^-/) {
            s/^-//;
            $self->set_value($_);
            $entry->set_position(0);
        } else {
            $entry->set_position(0);
        };
        return(1);
    };

    # CHARS "," and "."
    if(($eventkey->keyval eq '44') || ($eventkey->keyval eq '46') || ($eventkey->keyval eq '65454')) {
        $entry->set_position(length($entry->get_text)-$DECIMALS);
        return(1);
    };


    # cursor LEFT
    if($eventkey->keyval eq '65361') {
        if($entry->get_position) {
            $entry->set_position($entry->get_position-1);
        };
        return(1);
    };

    # cursor RIGHT
    if($eventkey->keyval eq '65363') {
        if($entry->get_position<length($entry->get_text)) {
            $entry->set_position($entry->get_position+1);
        };
        return(1);
    };
  
    # key HOME
    if($eventkey->keyval eq '65360') {
        $entry->set_position(0);
        return(1);
    };

    # key END
    if($eventkey->keyval eq '65367') {
        $entry->set_position(length($entry->get_text));
        return(1);
    };

    # key BS
    if($eventkey->keyval eq '65288') {
        if($entry->get_selection_bounds) {
            my $endsel=$entry->get_selection_bounds;
            my $oldtext=$entry->get_text;
            my $oldlength=length($oldtext);
            $entry->delete_selection;
            $_=$entry->get_text;
            s/,/\./;
            my $newlength=length($_);
            my $startsel=$endsel-($oldlength-$newlength);
            $_=substr($oldtext,0,$startsel);
            s/ //g;
            my $oldspaces=length($_);
            $oldspaces=$startsel-$oldspaces;
            if(((($oldlength-$endsel)>0) && ($oldlength-$endsel)<=$DECIMALS)) {
                my $zeros="";
                for(my $i=$DECIMALS;$i>($oldlength-$endsel);$i--) {
                    $zeros.="0";
                };
                $_=substr($entry->get_text,0,$startsel).".".$zeros.substr($entry->get_text,$startsel);
            } else {
                $_=$entry->get_text;
            };
            s/,/\./;
            
            if((!$_) || ($_ eq '-')) {
                $_="0";
            };
            s/ //g;
            $self->set_value($_);
            my $val=Tools::HumanNums(sprintf("%01.".$DECIMALS."f",$_));
            if($_ eq '0') {
                $entry->select_region(0,1);
            };
            if(($oldlength-$startsel)==($oldlength-$DECIMALS+1)) {
                $entry->set_position(length($entry->get_text)-$DECIMALS);
            } else {
                $_=substr($val,0,$startsel+1);
                $newlength=length($_);
                s/ //g;
                my $newspaces=$newlength-length($_);
                $entry->set_position($startsel-($oldspaces-$newspaces));
            };
        } else {
            my $pos=$entry->get_position;
            if($pos>0) {
                $_=$entry->get_text;
                my $oldlength=length($_);
                s/ //g;
                my $oldspaces=$oldlength-length($_);
                my $val="";
                if(($oldlength-$pos)!=$DECIMALS) {
                    $_=substr($entry->get_text,0,$entry->get_position-1);
                    $_.=substr($entry->get_text,$entry->get_position);
                    s/,/\./;
                    s/ //g;
                    $self->set_value($_);
                    $val=Tools::HumanNums(sprintf("%01.".$DECIMALS."f",$_));
                };
                if(($oldlength-$pos)<=($oldlength-$DECIMALS)) {
                    if($val ne "") {
                        my $newlength=length($val);
                        s/ //g;
                        my $newspaces=$newlength-length($_);
                        $entry->set_position($pos-1-($oldspaces-$newspaces));
                    } else {
                        $entry->set_position($pos-1);
                    };
                    if($entry->get_position==1) {
                        $entry->select_region(0,1);
                    };
                };
            };
        };
        return(1);
    };

    # key DEL
    if($eventkey->keyval eq '65535') {
        if($entry->get_selection_bounds) {
            my $endsel=$entry->get_selection_bounds;
            my $oldtext=$entry->get_text;
            my $oldlength=length($oldtext);
            $entry->delete_selection;
            $_=$entry->get_text;
            s/,/\./;
            my $newlength=length($_);
            my $startsel=$endsel-($oldlength-$newlength);
            $_=substr($oldtext,0,$startsel);
            s/ //g;
            my $oldspaces=length($_);
            $oldspaces=$startsel-$oldspaces;
            if(((($oldlength-$endsel)>0) && ($oldlength-$endsel)<=$DECIMALS)) {
                my $zeros="";
                for(my $i=$DECIMALS;$i>($oldlength-$endsel);$i--) {
                    $zeros.="0";
                };
                $_=substr($entry->get_text,0,$startsel).".".$zeros.substr($entry->get_text,$startsel);
            } else {
                $_=$entry->get_text;
            };
            s/,/\./;
            
            if((!$_) || ($_ eq '-')) {
                $_="0";
            };
            s/ //g;
            $self->set_value($_);
            my $val=Tools::HumanNums(sprintf("%01.".$DECIMALS."f",$_));
            if($_ eq '0') {
                $entry->select_region(0,1);
            };
            if(($oldlength-$startsel)==($oldlength-$DECIMALS+1)) {
                $entry->set_position(length($entry->get_text)-$DECIMALS);
            } else {
                $_=substr($val,0,$startsel+1);
                $newlength=length($_);
                s/ //g;
                my $newspaces=$newlength-length($_);
                $entry->set_position($startsel-($oldspaces-$newspaces));
            };
        } else {
            my $pos=$entry->get_position;
            $_=$entry->get_text;
            my $oldlength=length($_);
            if($pos<$oldlength) {
                s/ //g;
                my $oldspaces=$oldlength-length($_);
                my $val="";
                if(($oldlength-$pos)!=$DECIMALS+1) {
                    if(($oldlength-$pos)<=($DECIMALS)) {
                        $_=substr($entry->get_text,0,$entry->get_position);
                        $_.="0".substr($entry->get_text,$entry->get_position+1);                     
                    } else {
                        $_=substr($entry->get_text,0,$entry->get_position);
                        $_.=substr($entry->get_text,$entry->get_position+1);
                    };
                    s/,/\./;
                    s/ //g;
                    $self->set_value($_);
                    $val=Tools::HumanNums(sprintf("%01.".$DECIMALS."f",$_));
                };
                if(($oldlength-$pos)>$DECIMALS+1) {
                    if($val ne "") {
                        my $newlength=length($val);
                        s/ //g;
                        my $newspaces=$newlength-length($_);
                        if($newlength==$oldlength) {
                            $entry->set_position($pos+1);
                        } else {
                            $entry->set_position($pos);
                        };
                    };
                } else {
                    $entry->set_position($pos+1);
                };
            };
        };
        return(1);
    };

    
    # numbers
    my $key_num="";
    if(($eventkey->keyval eq '48') || ($eventkey->keyval eq '65456')) {
        $key_num="0";
    };

    if(($eventkey->keyval eq '49') || ($eventkey->keyval eq '65457')) {
        $key_num="1";
    };

    if(($eventkey->keyval eq '50') || ($eventkey->keyval eq '65458')) {
        $key_num="2";
    };

    if(($eventkey->keyval eq '51') || ($eventkey->keyval eq '65459')) {
        $key_num="3";
    };

    if(($eventkey->keyval eq '52') || ($eventkey->keyval eq '65460')) {
        $key_num="4";
    };

    if(($eventkey->keyval eq '53') || ($eventkey->keyval eq '65461')) {
        $key_num="5";
    };

    if(($eventkey->keyval eq '54') || ($eventkey->keyval eq '65462')) {
        $key_num="6";
    };

    if(($eventkey->keyval eq '55') || ($eventkey->keyval eq '65463')) {
        $key_num="7";
    };

    if(($eventkey->keyval eq '56') || ($eventkey->keyval eq '65464')) {
        $key_num="8";
    };

    if(($eventkey->keyval eq '57') || ($eventkey->keyval eq '65465')) {
        $key_num="9";
    };
    
    if($key_num ne "") {
        if($entry->get_selection_bounds) {
            my $endsel=$entry->get_selection_bounds;
            my $oldtext=$entry->get_text;
            my $oldlength=length($oldtext);
            $entry->delete_selection;
            $_=$entry->get_text;
            s/,/\./;
            my $newlength=length($_);
            my $startsel=$endsel-($oldlength-$newlength);
            $_=substr($oldtext,0,$startsel);
            s/ //g;
            my $oldspaces=length($_);
            $oldspaces=$startsel-$oldspaces;
            if(($oldlength-$startsel)==$DECIMALS+1) {
                $_=substr($entry->get_text,0,$startsel).".".$key_num;
            } else {
                if(((($oldlength-$endsel)>0) && ($oldlength-$endsel)<=$DECIMALS)) {
                    my $zeros="";
                    for(my $i=$DECIMALS;$i>($oldlength-$endsel);$i--) {
                        $zeros.="0";
                    };
                    $_=substr($entry->get_text,0,$startsel).$key_num.".".$zeros;
                } else {
                    $_=substr($entry->get_text,0,$startsel).$key_num;
                };
            };
            $_.=substr($entry->get_text,$startsel);
            s/,/\./;
            s/ //g;
            $self->set_value($_);
            my $val=Tools::HumanNums(sprintf("%01.".$DECIMALS."f",$_));
            if(($oldlength-$startsel)==$DECIMALS+1) {
                $entry->set_position($startsel+2);
            } else {
                if(($oldlength-$startsel)<=$DECIMALS) {
                    $entry->set_position($startsel+1);
                } else {
                    $_=substr($val,0,$startsel+1);
                    $newlength=length($_);
                    s/ //g;
                    my $newspaces=$newlength-length($_);
                    $entry->set_position($startsel+1-($oldspaces-$newspaces));
                };
            };
        } else {
            my $pos=$entry->get_position;
            my $oldlength=length($entry->get_text);
            if($pos<$oldlength) {
                if(($oldlength-$pos)<=$DECIMALS) {
                    $_=substr($entry->get_text,0,$pos).$key_num;
                    $_.=substr($entry->get_text,$pos+1);
                } else {
                    $_=$entry->get_text;
                    if(($pos==0) && /^-/) {
                        $_="-".$key_num.substr($entry->get_text,1);
                    } else {
                        $_=substr($entry->get_text,0,$pos).$key_num;
                        $_.=substr($entry->get_text,$pos);
                    };
                };
                s/,/\./;
                s/ //g;
                my $val=$_;
                $self->set_value($_);
                $_=$entry->get_text;
                if(($pos==0) && /^-/) {
                    $entry->set_position($pos+2);
                } else {
                    if(($oldlength-$pos)<=$DECIMALS) {
                        $_=length(Tools::HumanNums(sprintf("%01.".$DECIMALS."f",$val)))-$oldlength-1;
                        $entry->set_position($pos+2+$_);
                    } else {
                        $_=length(Tools::HumanNums(sprintf("%01.".$DECIMALS."f",$val)))-$oldlength-1;
                        $entry->set_position($pos+1+$_);
                    };
                };
            };
        };
        return(1);
    };
};


1;
